/***********************************************
MIDI Toy - Full MIDI Sketch
Author: Jason Webb
Date: 03/01/2012
URL: http://cs.unk.edu/~webb/

Description: Sketch meant to send MIDI note on and note off messages
every time pins go high or low, based on input from buttons.

Uses debouncing technique outlined by Limor "ladyada" Fried at:
http://www.adafruit.com/blog/2009/10/20/example-code-for-multi-button-checker-with-debouncing/

MIDI note values are chosen arbitrarily, and taken from the following table:
http://www.phys.unsw.edu.au/jw/notes.html

***********************************************/

// Array of all pins which are connected to buttons
byte buttons[]   = {PIN_B7, PIN_D0, PIN_D1, PIN_D4, PIN_D5, PIN_D6, PIN_D7, PIN_E0, PIN_E1, PIN_C0, PIN_C1, PIN_C2, PIN_C3, PIN_B3, PIN_B2, PIN_B1, PIN_B0, PIN_E7, PIN_E6, PIN_F0, PIN_F1, PIN_F2, 32, 33, 34, 28, 29, 30, 31};

// Array of all MIDI notes associated with above buttons
byte midiNotes[] = {40,     41,     42,     43,     45,     46,     47,     48,     49,     50,     51,     52,     53,     54,     55,     56,     57,     58,     59,     60,     61,     62,     63, 64, 65, 66, 67, 68, 69};

// MIDI channel number
byte channel = 1;

#define NUMBUTTONS sizeof(buttons)
#define DEBOUNCE 10  // button debouncer, how many ms to debounce, 5+ ms is usually plenty

volatile byte pressed[NUMBUTTONS], justpressed[NUMBUTTONS], justreleased[NUMBUTTONS];

void setup() {
  // Set up all input pins with internal pull-up
  for (byte i=0; i<NUMBUTTONS; i++)
    pinMode(buttons[i], INPUT_PULLUP);

  // Run timer2 interrupt every 15 ms 
  TCCR2A = 0;
  TCCR2B = 1<<CS22 | 1<<CS21 | 1<<CS20;

  //Timer2 Overflow Interrupt Enable
  TIMSK2 |= 1<<TOIE2;

}

void loop() {
  // Check the state of each button
  for (byte i=0; i<NUMBUTTONS; i++) {
    // Button has just been pressed
    if (justpressed[i]) {
      justpressed[i] = 0;
      usbMIDI.sendNoteOn(midiNotes[i], 99, channel);
    }
    
    // Button has just been released
    if (justreleased[i]) {
      justreleased[i] = 0;
      usbMIDI.sendNoteOff(midiNotes[i], 0, channel);
    }
  }
}

// Debounce code from Adafruit
SIGNAL(TIMER2_OVF_vect) {
  check_switches();
}

void check_switches()
{
  static byte previousstate[NUMBUTTONS];
  static byte currentstate[NUMBUTTONS];
  static long lasttime;
  byte index;

  if (millis() < lasttime) {
     // we wrapped around, lets just try again
     lasttime = millis();
  }

  if ((lasttime + DEBOUNCE) > millis()) {
    // not enough time has passed to debounce
    return;
  }
  // ok we have waited DEBOUNCE milliseconds, lets reset the timer
  lasttime = millis();

  for (index = 0; index < NUMBUTTONS; index++) {

    currentstate[index] = digitalRead(buttons[index]);   // read the button

    /*     
    Serial.print(index, DEC);
    Serial.print(": cstate=");
    Serial.print(currentstate[index], DEC);
    Serial.print(", pstate=");
    Serial.print(previousstate[index], DEC);
    Serial.print(", press=");
    */

    if (currentstate[index] == previousstate[index]) {
      if ((pressed[index] == LOW) && (currentstate[index] == LOW)) {
          // just pressed
          justpressed[index] = 1;
      }
      else if ((pressed[index] == HIGH) && (currentstate[index] == HIGH)) {
          // just released
          justreleased[index] = 1;
      }
      pressed[index] = !currentstate[index];  // remember, digital HIGH means NOT pressed
    }
    //Serial.println(pressed[index], DEC);
    previousstate[index] = currentstate[index];   // keep a running tally of the buttons
  }
}
